/* --------------------------------------------------------- */
/* Dispo Request Answers       V001.001 ---> 03.06.2008, und */
/* WebSphere MQ Client PUT application                       */
/* Requires: rxmqt.dll (Rexx-MQ-Routine)                     */
/* --------------------------------------------------------- */
/* Processes all files in input directory                    */
/* - read file into buffer                                   */
/* - split Anldeb file into single messages, each 160 lines  */
/* - write buffer to MQ queue                                */
/* --------------------------------------------------------- */
parse arg parms                 /* get the target queue name */

/* --------------------------------------------------------- */

call FileSpecs        /* input and output file specification */
call InitFields       /* init fields                         */
call InitFunctions    /* init MQ                             */
call ConnectToMq      /* connect to MQ and open queue        */
if rcOpen = 0 then call ProcessFiles
call EndProcedure     /* close files an terminate MQ         */
exit                  /* End                                 */
/* --------------------------------------------------------- */

/* --------------------------------------------------------- */
FileSpecs:
/* --------------------------------------------------------- */
  FileSpec   = "*.*"
  InpPath    = "C:\MQ-Tools\Input\"
  LogPath    = "C:\MQ-Tools\Log\"
  ArchivPath = "C:\MQ-Tools\Archiv\"

  log_file   = "DDUpload.log"
  logFile    = .stream~new(LogPath || log_file)
  do until log_open = "READY:"
    log_open = logFile~open
    if log_open <> "READY:"
    then do
      say "===> RC-LogOpen =" log_open
      say "===> ... waiting 5 sec..."
      Call SysSleep 5
    end
  end
  call SysFileTree InpPath || FileSpec, 'file', FO
  if file.0 = 0
  then do
    say "===> no Files found for DD-Upload:"
    pull .
    exit -9999  /* leave if directory is empty */
  end
  say "===> Files found for DD-Upload:" file.0
return FileSpecs

/* --------------------------------------------------------- */
InitFields:
/* --------------------------------------------------------- */
  if parms~length = 0
  then do
    say "===========================================================>"
    say "===> Enter QueueName for MQPUT and environment (MIL)"
    pull parms
  end
  if parms~length = 0 then exit -0815
  parms = parms~translate
  parse var parms queue_name mq_env
  if parms~length = 0 then exit -0815
  if mq_env = "" then mq_env = "MIL"
  if mq_env = "MIL"
  then do
    queue_mgr = "MQMIL2"
    "SET MQSERVER=MQMIL2T.CH/TCP/171.13.20.41(1415)"
  end
  say "===> QueueManager is" queue_mgr
  say "===========================================================>"
  start_message = "MAN" queue_mgr "===>" queue_name
  LogMsg = start_message 
  call WriteToLog
return InitFields

/* --------------------------------------------------------- */
InitFunctions:
/* --------------------------------------------------------- */
  call rxfuncadd 'sysloadfuncs', 'rexxutil', 'sysloadfuncs'
  call sysloadfuncs
  rcc = RxFuncadd('RXMQINIT','RXMQT','RXMQINIT')
  parse var rcc rcFunc .
  if rcFunc <> 0 & rcFunc <> 1
  then do
    LogMsg = "Error Load MQ Functions"
    call WriteToLog
    LogMsg = "-"~copies(80)
    call WriteToLog
    exit -3333
  end
  rcc = RXMQINIT()
  rcc = RXMQCONS()
  parse var rcc rcInit .
  if rcInit <> 0 & rcInit <> 1
  then do
    LogMsg = "Error Init MQ Functions"
    call WriteToLog
    LogMsg = "-"~copies(80)
    call WriteToLog
    exit -4444
  end
return InitFunctions

/* --------------------------------------------------------- */
ConnectToMq:
/* --------------------------------------------------------- */
  rcc = RXMQCONN(queue_mgr)
  parse var rcc rcConn .
  if rcConn = 0
  then do
    iod.ot = MQOT_Q
    rcc = RXMQOPEN(queue_name, mqoo_output, 'ObjHndl', 'ood.')
    parse var rcc rcOpen .
    if rcOpen <> 0
    then do
      LogMsg = "MQOPEN failed with" rcc
      call WriteToLog
      LogMsg = "-"~copies(80)
      call WriteToLog
      exit -5555
    end
  end
  else do
    LogMsg = "MQCONN failed with" rcc
    call WriteToLog
    LogMsg = "-"~copies(80)
    call WriteToLog
    exit -6666
  end
return ConnectToMq

/* --------------------------------------------------------- */
ProcessFiles:
/* --------------------------------------------------------- */
  do i = 1 to file.0              /* Process number of files */
    in_file      = file.i
    in_file_name = in_file~substr(in_file~lastpos("\") + 1)
    in_file_path = in_file~substr(1,in_file~lastpos("\"))
    in_file_noex = in_file_name~substr(1,in_file_name~lastpos(".") - 1)
    in_file_err  = in_file_noex || ".err" ; rcInc = ""
    in_file_inc  = in_file_path || in_file_noex || ".notsent" /* incomplete file */
    mq_inp = .stream~new(in_file)
    rcInp = mq_inp~open('read')
    mq_inc = .stream~new(in_file_inc)
    rcInc = mq_inc~open('write')
    if rcInp = 'READY:'
    then do
      lineCount = 0 ; g.1 = "" ; byteCount = 0 ; msgCount = 0
      do while mq_inp~lines > 0
        lBuffer = mq_inp~linein || "0D0A"x
        lineCount = lineCount + 1
        byteCount = byteCount + lBuffer~length
        g.1 = g.1 || lBuffer
        if lineCount = 117
        then do
          g.0 = g.1~length
          msgCount = msgCount + 1
          call MqPutRoutine
          g.1 = "" ; lineCount = 0
        end
      end
      call FinishProcess
      LogMsg = "File [" || in_file_name || "], Bytes [" || byteCount || "], Messages [" || msgCount || "]"
      call WriteToLog
    end
    if rcInp <> 'READY:' then call ErrOpenInp
  end
return ProcessFiles

/* --------------------------------------------------------- */
/* call putqueue routine                                     */
MqPutRoutine:
/* --------------------------------------------------------- */
  ipmd.PER = MQPER_PERSISTENT
  ipmo.opt = MQPMO_SYNCPOINT
  ipmd.form = 'MQSTR'
  rcc = RXMQPUT( ObjHndl,'g.','ipmd.','opmd.','ipmo.','opmo.')
  parse var rcc rcPut .
  if rcPut = 0
  then do
    rcc = RXMQCMIT()
    parse var rcc rcCmt .
    if rcCmt = 0 then nop
  end
  else do
    mq_inc~charout(g.1)
    say g.1~substr(13,4) rcPut rcInc
    LogMsg = "MQPUT failed with" rcc
    call WriteToLog
  end
return MqPutRoutine

/* --------------------------------------------------------- */
/* Clean up files, directory                                 */
FinishProcess:
/* --------------------------------------------------------- */
  mq_inp~close
  mq_inc~close
  if rcPut = 0
  then do
    rcDel = SysFileDelete(ArchivPath || in_file_name)
    if rcDel = 0 then say "===> Duplicate File in Arcive-Directory deleted:"
    rcDel = SysFileDelete(in_file_inc)
    rcMov = SysFileMove(in_file, ArchivPath || in_file_name)
    if rcMov <> 0
    then do
      LogMsg = "MOVE failed with RC" rc
  	  call WriteToLog
    end
  end
  else do
    rcRen = SysFileCopy(in_file, in_file_err)
    rcRen = SysFileDelete(in_file)
    if rcRen <> 0
    then do
      LogMsg = "RENAME failed with RC" rcRen
   	  call WriteToLog
    end
  end                            /* end filesize > 0         */
return FinishProcess

/* --------------------------------------------------------- */
/* Write to logfile                                          */
WriteToLog:
/* --------------------------------------------------------- */
  TimeStamp = "["date()~right(11) time()"] --->"
  if log_open <> "READY:" then exit -7777
  LogFile~lineout(TimeStamp LogMsg)
return WriteToLog

/* --------------------------------------------------------- */
EndProcedure:                 /* close files an terminate MQ */
/* --------------------------------------------------------- */
  LogMsg = "-"~copies(80)
  call WriteToLog
  LogFile~close
  rcc = RXMQCLOSE(ObjHndl, mqco_none)
  rcc = RXMQDISC()
return EndProcedure

/* --------------------------------------------------------- */
/* Error procedures                                          */
/* --------------------------------------------------------- */
ErrOpenInp:
  /* Input file cannot be opened or in use                   */
  LogMsg = "Cannot open Input file:" in_file_name "RC="rc
  call WriteToLog
return ErrOpenInp

EmptyFile:
  LogMsg = "Length of input file is zero ! file:" in_file_name
  call WriteToLog
return EmptyFile