/* ------------------------------------------------------------ */
/* MassPayment and DirectDebit Upload                           */
/*                                V001.001 ---> 06.03.2009, und */
/* WebSphere MQ PUT application                                 */
/* ------------------------------------------------------------ */
/* Requires: RXMQT.dll (Rexx-MQ-Routine-Client)                 */
/*           RXMQN.dll (Rexx-MQ-Routine-Server)                 */
/* WMQ Support Pac MA95. Version 1.0 (C)                        */
/* Copyright IBM Corporation. 1997, 2008.                       */
/* ------------------------------------------------------------ */
/* Processes files in input directory                           */
/* - split Anlpay, Anldeb bundle file into single messages of   */
/*   variable length                                            */
/* - message starts with ... the first byte                     */
/* - message ends with "{5:.....}" CRLF                         */
/* ------------------------------------------------------------ */
parse arg parms

/* ------------------------------------------------------------ */

call FileSpecs        /* input and output file specification    */
call InitFields       /* init fields                            */
call InitFunctions    /* init MQ                                */
call ConnectToMq      /* connect to MQ and open queue           */
if rcOpen = 0 then call ProcessFiles
call EndProcedure     /* close files an terminate MQ            */
exit                  /* End                                    */
/* ------------------------------------------------------------ */

/* ------------------------------------------------------------ */
FileSpecs:
/* ------------------------------------------------------------ */
  FileSpec   = "*.*"
  InpPath    = "C:\MQ-Tools\Input\"
  LogPath    = "C:\MQ-Tools\Log\"
  ArchivPath = "C:\MQ-Tools\Archiv\"

  log_file   = "MQUpload.log"
  logFile    = .stream~new(LogPath || log_file)
  do until log_open = "READY:"
    log_open = logFile~open
    if log_open <> "READY:"
    then do
      say "===> RC-LogOpen =" log_open
      say "===> ... waiting 5 sec..."
      Call SysSleep 5
    end
  end
  call SysFileTree InpPath || FileSpec, 'file', FO
  if file.0 = 0
  then do
    say "===> no Files found for MP-Upload:"
    pull .
    exit -9999  /* leave if directory is empty */
  end
  say "===> Files found for MP-Upload:" file.0
return FileSpecs

/* ------------------------------------------------------------ */
InitFields:
/* ------------------------------------------------------------ */
  if parms~length = 0
  then do
    say "===========================================================>"
    say "===> Enter QueueName for MQPUT and environment (TST/UAT/PRD/FFM)"
    pull parms
  end
  if parms~length = 0 then exit -0815
  parms = parms~translate
  parse var parms queue_name mq_env
  if mq_env = "" then mq_env = "FFM"
  if mq_env = "FFM"
  then do
    queue_mgr = "MQFFM01"    
    "SET MQSERVER=MQFFM01T.CH/TCP/ibfftocw22224.de.AD.drkw.net(1414)"
  end
  say "===========================================================>"
  LogMsg.1 = "Application: RxMQPUT , Target: QM =" queue_mgr", Queue =" queue_name
  LogMsg.2 = "-"~copies(80)
  LogMsg.0 = 1
  say "===>" LogMsg.1
  call WriteToLog
return InitFields

/* ------------------------------------------------------------ */
InitFunctions:
/* ------------------------------------------------------------ */
  call rxfuncadd 'sysloadfuncs', 'rexxutil', 'sysloadfuncs'
  call sysloadfuncs
  rcc = RxFuncadd('RXMQINIT','RXMQT','RXMQINIT')
  parse var rcc rcFunc .
  if rcFunc <> 0 & rcFunc <> 1
  then do
    LogMsg.0 = 2
    LogMsg.1 = "Error Load MQ Functions"
    call WriteToLog
    exit -3333
  end
  rcc = RXMQINIT()
  rcc = RXMQCONS()
  parse var rcc rcInit .
  if rcInit <> 0 & rcInit <> 1
  then do
    LogMsg.0 = 2
    LogMsg.1 = "Error Init MQ Functions"
    call WriteToLog
    exit -4444
  end
return InitFunctions

/* ------------------------------------------------------------ */
ConnectToMq:
/* ------------------------------------------------------------ */
  rcc = RXMQCONN(queue_mgr)
  parse var rcc rcConn .
  if rcConn = 0
  then do
    iod.ot = MQOT_Q
    rcc = RXMQOPEN(queue_name, mqoo_output, 'ObjHndl', 'ood.')
    parse var rcc rcOpen .
    if rcOpen <> 0
    then do
      LogMsg.0 = 2
      LogMsg.1 = "MQOPEN failed with" rcc
      call WriteToLog
      exit -5555
    end
  end
  else do
    LogMsg.0 = 2
    LogMsg.1 = "MQCONN failed with" rcc
    call WriteToLog
    exit -6666
  end
return ConnectToMq

/* ------------------------------------------------------------ */
ProcessFiles:
/* ------------------------------------------------------------ */
  do i = 1 to file.0                 /* Process number of files */
    in_file      = file.i
    in_file_name = in_file~substr(in_file~lastpos("\") + 1)
    in_file_path = in_file~substr(1,in_file~lastpos("\"))
    if in_file_name~lastpos(".") > 0
    then in_file_noex = in_file_name~substr(1,in_file_name~lastpos(".") - 1)
    else in_file_noex = in_file_name
    in_file_err  = in_file_noex || ".err" ; rcInc = ""
    in_file_inc  = in_file_path || in_file_noex || ".notsent" /* incomplete file */
    mq_inp = .stream~new(in_file)
    rcInp = mq_inp~open('read')
    mq_inc = .stream~new(in_file_inc)
    rcInc = mq_inc~open('write')
    if rcInp = 'READY:'
    then do
      call VerifyFile
      if msgTable.0 > 0
      then do j = 1 to msgTable.0
        parse var msgTable.j BuffPntr BuffMsgLen
        call MqPutRoutine
      end
      LogMsg.0 = 1
      LogMsg.1 = "MQPUT ["in_file_name"] completed with rc" rcPut". BundleSize:" putCount "of" msgTable.0", BytesTotal:" byteCount
      call WriteToLog
      call FinishProcess
    end
    if rcInp <> 'READY:'
    then do
      LogMsg.0 = 2
      LogMsg.1 = "Error open Input file" in_file_name
      call WriteToLog
      exit -4444
    end
  end
return ProcessFiles

/* ------------------------------------------------------------ */
/* Get Message positions                                        */
VerifyFile:
/* ------------------------------------------------------------ */
  BufferLength = mq_inp~chars
  MsgBuffer = mq_inp~charin(,BufferLength)
  msgStart = 1 ; msgEnd = BufferLength ; msgNext = 1 
  msgCount = 0 ; EndOfFile = 0 ; findEndChar = 0
  putCount = 0 ; byteCount = 0
  if BufferLength > 0
  then do j = 1 to  BufferLength
    if MsgBuffer~substr(j,7) = "-}" || "0D0A"x || "{5:"
    then findEndChar = 1
    if findEndChar > 0
    then do
      if findEndChar > 7          /* search after 7 chars later */
      then do
        if MsgBuffer~substr(j,3) = "}" || "0D0A"x
        then do
          msgCount = msgCount + 1
          msgEnd = j + 3
          msgLen = msgEnd - msgStart
          msgTable.msgCount = msgStart msgLen
          msgStart = j + 3                          /* add CRLF */
          findEndChar = 0
        end
      end
      findEndChar = findEndChar + 1
    end
  end
  msgTable.0 = msgCount
return VerifyFile

/* ------------------------------------------------------------ */
/* call putqueue routine                                        */
MqPutRoutine:
/* ------------------------------------------------------------ */
  g.1 = MsgBuffer~substr(BuffPntr,BuffMsgLen)
  g.0 = BuffMsgLen
  ipmd.PER = MQPER_PERSISTENT
  ipmo.opt = MQPMO_SYNCPOINT
  ipmd.form = 'MQSTR'
  rcc = RXMQPUT( ObjHndl,'g.','ipmd.','opmd.','ipmo.','opmo.')
  parse var rcc rcPut .
  if rcPut = 0
  then do
    rcc = RXMQCMIT()
    parse var rcc rcCmt .
    if rcCmt = 0
    then do
      putCount = putCount + 1
      byteCount = byteCount + BuffMsgLen
    end
  end
  else do
    mq_inc~charout(g.1)
    say g.1~substr(13,4) rcPut rcInc
    LogMsg.0 = 1
    LogMsg.1 = "MQPUT failed with" rcc
    call WriteToLog
  end
return MqPutRoutine

/* ------------------------------------------------------------ */
/* Clean up files, directory                                    */
FinishProcess:
/* ------------------------------------------------------------ */
  LogMsg = "Processing file" in_file
  mq_inp~close
  mq_inc~close
  if rcPut = 0 & msgCount = putCount
  then do
    rcDel = SysFileDelete(ArchivPath || in_file_name)
    if rcDel = 0 then say "===> Duplicate File in Arcive-Directory deleted:"
    rcDel = SysFileDelete(in_file_inc)
    rcMov = SysFileMove(in_file, ArchivPath || in_file_name)
    if rcMov <> 0
    then do
      LogMsg.0 = 1
      LogMsg.1 = "MOVE failed with RC" rc
  	  call WriteToLog
    end
	end
  else do
    rcRen = SysFileCopy(in_file, in_file_err)
    rcRen = SysFileDelete(in_file)
    if rcRen <> 0
    then do
      LogMsg.0 = 1
      LogMsg.1 = "RENAME failed with RC" rcRen
   	  call WriteToLog
    end
  end                               /* end filesize > 0         */
return FinishProcess

/* ------------------------------------------------------------ */
/* Write to logfile                                             */
WriteToLog:
/* ------------------------------------------------------------ */
  TimeStamp = "["date()~right(11) time()"] --->"
  if log_open <> "READY:" then exit -7777
  if LogMsg.0 > 0
  then do LogCount = 1 to LogMsg.0
    LogFile~lineout(TimeStamp LogMsg.LogCount)
  end
return WriteToLog

/* ------------------------------------------------------------ */
EndProcedure:                    /* close files an terminate MQ */
/* ------------------------------------------------------------ */
  LogMsg.0 = 1
  LogMsg.1 = LogMsg.2
  call WriteToLog
  LogFile~close
  rcc = RXMQCLOS(ObjHndl, mqco_none)
  rcc = RXMQDISC()
return EndProcedure
