/* ------------------------------------------------------------ */
/* MqFileDown V001.001 --->                     20.03.2009, und */
/* WebSphere MQ GET application                                 */
/* ------------------------------------------------------------ */
/* Requires: RXMQT.dll (Rexx-MQ-Routine-Client) <-------------- */
/*           RXMQN.dll (Rexx-MQ-Routine-Server)                 */
/* WMQ Support Pac MA95. Version 1.0 (C)                        */
/* Copyright IBM Corporation. 1997, 2008.                       */
/* ------------------------------------------------------------ */
/* - Read until end-of-queue                                    */
/* - Write message to file                                      */
/* ------------------------------------------------------------ */
parse arg qmgrName startParms

/* ------------------------------------------------------------ */
call InitFields          /* init fields                         */
call InitFunctions       /* init MQ                             */
call ConnectToMq         /* connect to MQ and open queue        */
if rcOpen = 0 then call ProcessQueue
call EndProcedure        /* close files an terminate MQ         */
exit                     /* End                                 */
/* ------------------------------------------------------------ */

/* ------------------------------------------------------------ */
InitFields:
/* ------------------------------------------------------------ */
  LogMsg.2 = "-"~copies(80)
  psLength = startParms~length
  parse value time("L") with hh ":" mm ":" ss "." ms
  FileTime = hh || mm || ss
  FileDate = date("S")~substr(3)
  LogPath    = "C:\MQ-Tools\Log\"
  ArchivPath = "C:\MQ-Tools\Done\"
  fileSuff = ".dat"
  log_file   = "MQDownload.log"
  logFile = .stream~new(LogPath || log_file)
  do until logOpen = "READY:"
    logOpen = logFile~open
    if logOpen <> "READY:"
    then do
      /* ===> ... waiting 5 sec... */
      Call SysSleep 5
    end
  end

  if qmgrName~length = 0 & psLength = 0
  then do
    say "===================================================================>"
    say "===> Enter QueueManager QueueName and ProcessName for MQGET"
    pull parms
    if parms~length = 0 then exit -0815
    parms = parms~translate
    parse var parms qmgrName queueName processName
    startMessage = "MAN"
  end
  if psLength > 0
  then do
    if startParms~substr(3,3) = "TMC"             /* eat the trigger */
    then do
      startMessage = "TMC"
      queueName = startParms~substr(11,48)   ; queueName = queueName~space
      processName = startParms~substr(59,48) ; processName = processName~space
    end
    else do
      startMessage = "ERR"
      LogMsg.1 = "Application : RxMQGET , Target: QM =" qmgrName", ERROR: Invalid Trigger Message Received !"
      LogMsg.0 = 2
      call WriteToLog
      exit -0815
    end
  end

  LogMsg.1 = "Application : RxMQGET , Target: QM =" qmgrName", Queue =" queueName "StartType =" startMessage
  LogMsg.0 = 1
  if startMessage = "MAN"
  then do
    say "===>" LogMsg.1
    say "---> Start ====> :" time()
  end
  call WriteToLog
  msgCount = 0 ; byteCount = 0
  
  select
    when processName = "UNTERKD"
    then do
      OutPath    = "C:\MQ-Tools\Output\"
      filePref = "CCM" || FileDate
    end
    otherwise do
      OutPath    = "C:\MQ-Tools\Output\"
      filePref = FileDate
    end
  end
  OutFile   = FilePref || fileSuff
  OutStream = .stream~new(OutPath || OutFile)

return InitFields

/* ------------------------------------------------------------ */
InitFunctions:
/* ------------------------------------------------------------ */
  call rxfuncadd 'sysloadfuncs', 'rexxutil', 'sysloadfuncs'
  call sysloadfuncs
  rcc = RxFuncadd('RXMQINIT','RXMQT','RXMQINIT')
  parse var rcc rcFunc .
  if rcFunc <> 0 & rcFunc <> 1
  then do
    LogMsg.0 = 2
    LogMsg.1 = "Error Load MQ Functions"
    call WriteToLog
    exit -3333
  end
  rcc = RXMQINIT()
  rcc = RXMQCONS()
  parse var rcc rcInit .
  if rcInit <> 0 & rcInit <> 1
  then do
    LogMsg.0 = 2
    LogMsg.1 = "Error Init MQ Functions"
    call WriteToLog
    exit -4444
  end
Return InitFunctions

/* ------------------------------------------------------------ */
ConnectToMq:
/* ------------------------------------------------------------ */
  rcc = RXMQCONN(qmgrName)
  parse var rcc rcConn .
  if rcConn = 0
  then do
    iod.ot = MQOT_Q
    rcc = RXMQOPEN(queueName, mqoo_input_shared, 'ObjHndl', 'ood.')
    parse var rcc rcOpen .
    if rcOpen <> 0
    then do
      LogMsg.0 = 2
      LogMsg.1 = "MQOPEN failed with" rcc
      call WriteToLog
      exit -4444
    end
  end
  else do
    LogMsg.0 = 2
    LogMsg.1 = "MQCONN failed with" rcc
    call WriteToLog
    exit -4444
  end
return ConnectToMq

/* ------------------------------------------------------------ */
ProcessQueue:
/* ------------------------------------------------------------ */
  igmo.opt = MQGMO_WAIT + MQGMO_SYNCPOINT + MQGMO_FAIL_IF_QUIESCING + MQGMO_ACCEPT_TRUNCATED_MSG
  igmo.wait = 10000
  
  rcGet = 0 ; msgCount = 0 ; byteCount = 0
  do until rcGet <> 0
    g.0 = 4194304 ; g.1 = ''
    rcc = RXMQGET( ObjHndl,'g.','igmd.','ogmd.','igmo.','ogmo.')
    parse var rcc rcGet .
    if rcGet = 0
    then do
      call MessageToFile
      msgCount = msgCount + 1
      byteCount = byteCount + g.0
    end
  end
  if rcGet = 2033 then rcGet = "2033[End-of-Queue]"
  LogMsg.0 = 1
  LogMsg.1 = "MQGET ["OutFile"] completed with rc" rcGet". Msgs:" msgCount ", Bytes:" byteCount
  call WriteToLog
  if startMessage = "MAN" then say "---> Processed   :" msgCount
return ProcessQueue

MessageToFile:
/* --------------------------------------------------------- */
/* Write to file                                             */
/* --------------------------------------------------------- */
  rcWrite = OutStream~lineout(g.1)
  if rcWrite = 0
  then do
    rcc = RXMQCMIT()
    parse var rcc rcCmt .
    if rcCmt <> 0
    then do
      if startMessage = "MAN" then say "---> Commit Err  :" rcc
      LogMsg.0 = 1
      LogMsg.1 = "Commit failed with" rcc
      call WriteToLog
    end
  end
  else do
    rcc = RXMQBACK()
    parse var rcc rcBck .
    if rcBck <> 0
    then do
      if startMessage = "MAN" then say "---> Backout Err :" rcc
      LogMsg.0 = 1
      LogMsg.1 = "Backout failed with" rcc
      call WriteToLog
    end
    LogMsg.0 = 2
    LogMsg.1 = "Backout required due to io error" rcWrite
    call WriteToLog
    exit -9898
  end
return MessageToFile

/* --------------------------------------------------------- */
/* Write to logfile                                          */
WriteToLog:
/* --------------------------------------------------------- */
  TimeStamp = "["date()~right(11) time()"] --->"
  if logOpen <> "READY:" then exit -7777
  if LogMsg.0 > 0
  then do LogCount = 1 to LogMsg.0
    LogFile~lineout(TimeStamp LogMsg.LogCount)
  end
return WriteToLog

/* ------------------------------------------------------------ */
EndProcedure:
/* ------------------------------------------------------------ */
  LogMsg.0 = 1
  LogMsg.1 = LogMsg.2
  call WriteToLog
  LogFile~close
  OutStream~close
  if rcOpen = 0
  then do
    rcc = RXMQCLOS(ObjHndl, mqco_none)
    parse var rcc rcClose .
  end
  if rcConn = 0
  then do
    rcc = RXMQDISC()
    parse var rcc rcDisc .
  end
return EndProcedure
